@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1997-2019, University of Amsterdam
+    Copyright (c)  1997-2020, University of Amsterdam
                               VU University Amsterdam
                               CWI, Amsterdam
     All rights reserved.
@@ -90,9 +90,11 @@ translate_message2(Term) -->
 translate_message2(Term) -->
     prolog_message(Term).
 translate_message2(error(resource_error(stack), Context)) -->
+    !,
     out_of_stack(Context).
-translate_message2(error(resource_error(Missing), _)) -->
-    [ 'Not enough resources: ~w'-[Missing] ].
+translate_message2(error(resource_error(tripwire(Wire, Context)), _)) -->
+    !,
+    tripwire_message(Wire, Context).
 translate_message2(error(ISO, SWI)) -->
     swi_location(SWI),
     term_message(ISO),
@@ -122,6 +124,8 @@ term_message(Term) -->
 term_message(Term) -->
     [ 'Unknown error term: ~p'-[Term] ].
 
+iso_message(resource_error(Missing)) -->
+    [ 'Not enough resources: ~w'-[Missing] ].
 iso_message(type_error(evaluable, Actual)) -->
     { callable(Actual) },
     [ 'Arithmetic: `~p'' is not a function'-[Actual] ].
@@ -434,6 +438,8 @@ swi_location(stream(Stream, Line, LinePos, CharNo)) -->
     ->  swi_location(file(File, Line, LinePos, CharNo))
     ;   [ 'Stream ~w:~d:~d '-[Stream, Line, LinePos] ]
     ).
+swi_location(autoload(File:Line)) -->
+    [ '~w:~w: '-[File, Line] ].
 swi_location(_) -->
     [].
 
@@ -930,6 +936,20 @@ prolog_message(autoload(Pred, File)) -->
     [ 'autoloading ~p from ~w'-[Pred, File] ].
 prolog_message(autoload(read_index(Dir))) -->
     [ 'Loading autoload index for ~w'-[Dir] ].
+prolog_message(autoload(disabled(Loaded))) -->
+    [ 'Disabled autoloading (loaded ~D files)'-[Loaded] ].
+
+swi_message(autoload(Msg)) -->
+    [ nl, '    ' ],
+    autoload_message(Msg).
+
+autoload_message(not_exported(PI, Spec, _FullFile, _Exports)) -->
+    [ ansi(code, '~w', [Spec]),
+      ' does not export ',
+      ansi(code, '~p', [PI])
+    ].
+autoload_message(no_file(Spec)) -->
+    [ ansi(code, '~p', [Spec]), ': No such file' ].
 
 
                  /*******************************
@@ -1255,9 +1275,16 @@ delays(_, _Options) -->
 
 list_clauses([]).
 list_clauses([H|T]) :-
-    portray_clause(user_output, H, [indent(4)]),
+    (   system_undefined(H)
+    ->  true
+    ;   portray_clause(user_output, H, [indent(4)])
+    ),
     list_clauses(T).
 
+system_undefined((undefined :- tnot(undefined))).
+system_undefined((answer_count_restraint :- tnot(answer_count_restraint))).
+system_undefined((radial_restraint :- tnot(radial_restraint))).
+
 bind_res_sep(_, []) --> !.
 bind_res_sep(_, []-[]) --> !.
 bind_res_sep([], _) --> !.
@@ -1547,6 +1574,25 @@ deprecated(set_prolog_stack(_Stack,limit)) -->
       'See https://www.swi-prolog.org/changes/stack-limit.html'
     ].
 
+		 /*******************************
+		 *           TRIPWIRES		*
+		 *******************************/
+
+tripwire_message(Wire, Context) -->
+    [ 'Trapped tripwire ~w for '-[Wire] ],
+    tripwire_context(Wire, Context).
+
+tripwire_context(_, ATrie) -->
+    { '$is_answer_trie'(ATrie),
+      !,
+      '$tabling':atrie_goal(ATrie, QGoal),
+      user_predicate_indicator(QGoal, Goal)
+    },
+    [ '~p'-[Goal] ].
+tripwire_context(_, Ctx) -->
+    [ '~p'-[Ctx] ].
+
+
 		 /*******************************
 		 *        DEFAULT THEME		*
 		 *******************************/
@@ -1596,6 +1642,19 @@ default_theme(message(Level),         Attrs) :-
 %   system.
 
 print_message(Level, Term) :-
+    nb_current('$inprint_message', true),
+    (   Level \== silent
+    ->  format(user_error, 'Recursive ~w message: ~q~n', [Level, Term])
+    ;   true
+    ),
+    !.
+print_message(Level, Term) :-
+    setup_call_cleanup(
+        nb_setval('$inprint_message', true),
+        print_message_guarded(Level, Term),
+        nb_delete('$inprint_message')).
+
+print_message_guarded(Level, Term) :-
     (   must_print(Level, Term)
     ->  (   translate_message(Term, Lines, [])
         ->  (   nonvar(Term),
