@@ -37,9 +37,13 @@
           [ ansi_format/3,              % +Attr, +Format, +Args
             ansi_get_color/2            % +Which, -rgb(R,G,B)
           ]).
-:- use_module(library(apply)).
-:- use_module(library(lists)).
-:- use_module(library(error)).
+:- autoload(library(apply),[maplist/3]).
+:- autoload(library(error),[domain_error/2,must_be/2]).
+:- autoload(library(lists),[flatten/2,append/2,append/3]).
+:- if(exists_source(library(time))).
+:- autoload(library(time),[call_with_time_limit/2]).
+:- endif.
+
 
 /** <module> Print decorated text to ANSI consoles
 
@@ -341,7 +345,7 @@ prolog:message_line_element(S, begin(Level, Ctx)) :-
     current_prolog_flag(color_term, true),
     !,
     (   is_list(Attr)
-    ->  maplist(sgr_code, Attr, Codes),
+    ->  sgr_codes(Attr, Codes),
         atomic_list_concat(Codes, ;, Code)
     ;   sgr_code(Attr, Code)
     ),
@@ -352,6 +356,11 @@ prolog:message_line_element(S, end(Ctx)) :-
     Ctx = ansi(Reset, _),
     keep_line_pos(S, write(S, Reset)).
 
+sgr_codes([], []).
+sgr_codes([H0|T0], [H|T]) :-
+    sgr_code(H0, H),
+    sgr_codes(T0, T).
+
 level_attrs(Level,         Attrs) :-
     user:message_property(Level, color(Attrs)),
     !.
@@ -397,6 +406,7 @@ keep_line_pos(_, G) :-
 %   integers in the range 0..65535.
 
 
+:- if(current_predicate(call_with_time_limit/2)).
 ansi_get_color(Which0, RGB) :-
     stream_property(user_input, tty(true)),
     stream_property(user_output, tty(true)),
@@ -480,6 +490,12 @@ echo([H|T]) :-
     put_code(user_output, H),
     echo(T).
 
+:- else.
+ansi_get_color(_Which0, _RGB) :-
+    fail.
+:- endif.
+
+
 
 :- multifile prolog:message//1.
 
