@@ -37,12 +37,19 @@
           [ safe_goal/1,                % :Goal
             safe_call/1                 % :Goal
           ]).
-:- use_module(library(assoc)).
-:- use_module(library(lists)).
-:- use_module(library(debug)).
-:- use_module(library(error)).
-:- use_module(library(prolog_format)).
-:- use_module(library(apply)).
+:- autoload(library(apply),[maplist/2]).
+:- autoload(library(assoc),[empty_assoc/1,get_assoc/3,put_assoc/4]).
+:- autoload(library(debug),[debug/3,debugging/1]).
+:- autoload(library(error),
+	    [ must_be/2,
+	      instantiation_error/1,
+	      type_error/2,
+	      permission_error/3
+	    ]).
+:- autoload(library(lists),[append/3]).
+:- autoload(library(prolog_format),[format_types/2]).
+:- use_module(library(apply_macros),[expand_phrase/2]).
+
 
 :- multifile
     safe_primitive/1,               % Goal
@@ -540,7 +547,9 @@ safe_primitive(=:=(_,_)).
 safe_primitive(=\=(_,_)).
 safe_primitive(=<(_,_)).
 safe_primitive(<(_,_)).
+:- if(current_prolog_flag(bounded, false)).
 safe_primitive(system:nth_integer_root_and_remainder(_,_,_,_)).
+:- endif.
 
                                         % term-handling
 safe_primitive(arg(_,_,_)).
@@ -853,6 +862,7 @@ safe_meta('$tabling':abolish_table_subgoals(V), []) :-
 safe_meta('$tabling':current_table(V, _), []) :-
     \+ qualified(V).
 safe_meta('$tabling':tnot(G), [G]).
+safe_meta('$tabling':not_exists(G), [G]).
 
 qualified(V) :-
     nonvar(V),
@@ -1218,13 +1228,10 @@ safe_directive_flag(back_quotes, _).
 %   and we only need to be  careful   if  the sandboxed code defines
 %   expansion rules.
 
-prolog:sandbox_allowed_expansion(Directive) :-
-    prolog_load_context(module, M),
-    debug(sandbox(expansion), 'Expand in ~p: ~p', [M, Directive]),
-    fail.
 prolog:sandbox_allowed_expansion(M:G) :-
     prolog_load_context(module, M),
     !,
+    debug(sandbox(expansion), 'Expand in ~p: ~p', [M, G]),
     safe_goal(M:G).
 prolog:sandbox_allowed_expansion(_,_).
 
