@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1995-2018, University of Amsterdam
+    Copyright (c)  1995-2020, University of Amsterdam
                               VU University Amsterdam
                               CWI, Amsterdam
     All rights reserved.
@@ -47,7 +47,9 @@
 
             win_add_dll_directory/1     % +Dir
           ]).
-:- use_module(library(lists), [reverse/2]).
+:- autoload(library(error),[existence_error/2,domain_error/2]).
+:- autoload(library(lists),[member/2,reverse/2]).
+
 :- set_prolog_flag(generate_debug_info, false).
 
 /** <module> Utility library for loading foreign objects (DLLs, shared objects)
@@ -107,9 +109,7 @@ predicate defined in C.
 
 :- meta_predicate
     load_foreign_library(:),
-    load_foreign_library(:, +),
-    use_foreign_library(:),
-    use_foreign_library(:, +).
+    load_foreign_library(:, +).
 
 :- dynamic
     loading/1,                      % Lib
@@ -136,6 +136,29 @@ predicate defined in C.
                       [ keep(true) ]).
 
 
+%!  use_foreign_library(+FileSpec) is det.
+%!  use_foreign_library(+FileSpec, +Entry:atom) is det.
+%
+%   Load and install a foreign   library as load_foreign_library/1,2 and
+%   register the installation using  initialization/2   with  the option
+%   `now`. This is similar to using:
+%
+%   ```
+%   :- initialization(load_foreign_library(foreign(mylib))).
+%   ```
+%
+%   but using the initialization/1 wrapper  causes   the  library  to be
+%   loaded _after_ loading of the file in which it appears is completed,
+%   while use_foreign_library/1 loads the   library  _immediately_. I.e.
+%   the difference is only relevant if the   remainder  of the file uses
+%   functionality of the C-library.
+%
+%   As of SWI-Prolog 8.1.22, use_foreign_library/1,2 is in provided as a
+%   built-in predicate that, if necessary,   loads  library(shlib). This
+%   implies that these directives can be used without explicitly loading
+%   library(shlib) or relying on demand loading.
+
+
                  /*******************************
                  *           DISPATCHING        *
                  *******************************/
@@ -210,8 +233,8 @@ lib_to_file(Lib, Lib, false).
 
 open_foreign_in_resources(Zipper, ForeignSpecAtom, Stream) :-
     term_to_atom(foreign(Name), ForeignSpecAtom),
-    zipper_members(Zipper, Entries),
-    entries_for_name(Name, Entries, Entries1),
+    zipper_members_(Zipper, Entries),
+    entries_for_name(Entries, Name, Entries1),
     compatible_architecture_lib(Entries1, Name, CompatibleLib),
     zipper_goto(Zipper, file(CompatibleLib)),
     zipper_open_current(Zipper, Stream,
@@ -219,6 +242,26 @@ open_foreign_in_resources(Zipper, ForeignSpecAtom, Stream) :-
                           release(true)
                         ]).
 
+%!  zipper_members_(+Zipper, -Members) is det.
+%
+%   Simplified version of zipper_members/2 from library(zip). We already
+%   have a lock  on  the  zipper  and   by  moving  this  here  we avoid
+%   dependency on another library.
+%
+%   @tbd: should we cache this?
+
+zipper_members_(Zipper, Members) :-
+    zipper_goto(Zipper, first),
+    zip_members__(Zipper, Members).
+
+zip_members__(Zipper, [Name|T]) :-
+    zip_file_info_(Zipper, Name, _Attrs),
+    (   zipper_goto(Zipper, next)
+    ->  zip_members__(Zipper, T)
+    ;   T = []
+    ).
+
+
 %!  compatible_architecture_lib(+Entries, +Name, -CompatibleLib) is det.
 %
 %   Entries is a list of entries  in   the  zip  file, which are already
@@ -258,6 +301,15 @@ qsave_compat_arch1(Arch1, Arch2) :-
 
 qsave:compat_arch(A,A).
 
+entries_for_name([], _, []).
+entries_for_name([H0|T0], Name, [H|T]) :-
+    shlib_atom_to_term(H0, H),
+    match_filespec(Name, H0),
+    !,
+    entries_for_name(T0, Name, T).
+entries_for_name([_|T0], Name, T) :-
+    entries_for_name(T0, Name, T).
+
 shlib_atom_to_term(Atom, shlib(Arch, Name)) :-
     sub_atom(Atom, 0, _, _, 'shlib('),
     !,
@@ -266,10 +318,6 @@ shlib_atom_to_term(Atom, Atom).
 
 match_filespec(Name, shlib(_,Name)).
 
-entries_for_name(Name, Entries, Filtered) :-
-    maplist(shlib_atom_to_term, Entries, Entries1),
-    include(match_filespec(Name), Entries1, Filtered).
-
 base(Path, Base) :-
     atomic(Path),
     !,
@@ -371,29 +419,6 @@ delete_foreign_lib(true, Path) :-
 delete_foreign_lib(_, _).
 
 
-%!  use_foreign_library(+FileSpec) is det.
-%!  use_foreign_library(+FileSpec, +Entry:atom) is det.
-%
-%   Load and install a foreign   library as load_foreign_library/1,2
-%   and register the installation using   initialization/2  with the
-%   option =now=. This is similar to using:
-%
-%     ==
-%     :- initialization(load_foreign_library(foreign(mylib))).
-%     ==
-%
-%   but using the initialization/1 wrapper causes  the library to be
-%   loaded _after_ loading of  the  file   in  which  it  appears is
-%   completed,  while  use_foreign_library/1  loads    the   library
-%   _immediately_. I.e. the  difference  is   only  relevant  if the
-%   remainder of the file uses functionality of the C-library.
-
-use_foreign_library(FileSpec) :-
-    initialization(load_foreign_library(FileSpec), now).
-
-use_foreign_library(FileSpec, Entry) :-
-    initialization(load_foreign_library(FileSpec, Entry), now).
-
 %!  unload_foreign_library(+FileSpec) is det.
 %!  unload_foreign_library(+FileSpec, +Exit:atom) is det.
 %
