@@ -366,7 +366,17 @@ reply_file_objects(File, Objs0, Objects, FileOptions, Options) :-
     module_info(File, ModuleOptions, Options),
     file_info(Objs0, Objs1, FileOptions, ModuleOptions),
     doc_hide_private(Objs1, ObjectsSelf, ModuleOptions),
-    include_reexported(ObjectsSelf, Objects, File, FileOptions).
+    include_reexported(ObjectsSelf, Objects1, File, FileOptions),
+    remove_doc_duplicates(Objects1, Objects, []).
+
+remove_doc_duplicates([], [], _).
+remove_doc_duplicates([H|T0], [H|T], Seen) :-
+    H = doc(_, _, Comment),
+    \+ memberchk(Comment, Seen),
+    !,
+    remove_doc_duplicates(T0, T, [Comment|Seen]).
+remove_doc_duplicates([_|T0], T, Seen) :-
+    remove_doc_duplicates(T0, T, Seen).
 
 include_reexported(SelfObjects, Objects, File, Options) :-
     option(include_reexported(true), Options),
@@ -1035,10 +1045,17 @@ object_synopsis(Name/Arity, Options) -->
     object_synopsis(_:Name/Arity, Options).
 object_synopsis(M:Name/Arity, Options) -->
     { functor(Head, Name, Arity),
-      predicate_property(M:Head, exported),
-      \+ predicate_property(M:Head, imported_from(_)),
-      module_property(M, file(File)),
-      file_name_on_path(File, Spec),
+      (   option(source(Spec), Options)
+      ->  absolute_file_name(Spec, File,
+                             [ access(read),
+                               file_type(prolog),
+                               file_errors(fail)
+                             ])
+      ;   predicate_property(M:Head, exported),
+          \+ predicate_property(M:Head, imported_from(_)),
+          module_property(M, file(File)),
+          file_name_on_path(File, Spec)
+      ),
       !,
       unquote_filespec(Spec, Unquoted),
       (   predicate_property(Head, autoload(FileBase)),
@@ -1051,6 +1068,14 @@ object_synopsis(M:Name/Arity, Options) -->
     ->  synopsis([code([':- use_module(',a(href(HREF), '~q'-[Unquoted]),').'])|Extra])
     ;   synopsis([code(':- use_module(~q).'-[Unquoted])|Extra])
     ).
+object_synopsis(Name//Arity, Options) -->
+    !,
+    { DCGArity is Arity+2 },
+    object_synopsis(Name/DCGArity, Options).
+object_synopsis(Module:Name//Arity, Options) -->
+    !,
+    { DCGArity is Arity+2 },
+    object_synopsis(Module:Name/DCGArity, Options).
 object_synopsis(f(_/_), _) -->
     synopsis(span(class(function),
                   [ 'Arithmetic function (see ',
@@ -1078,6 +1103,7 @@ synopsis(Text) -->
 unquote_filespec(Spec, Unquoted) :-
     compound(Spec),
     Spec =.. [Alias,Path],
+    atom(Path),
     atomic_list_concat(Parts, /, Path),
     maplist(need_no_quotes, Parts),
     !,
@@ -1928,8 +1954,8 @@ pred_source_href(Name/Arity, Module, HREF) :-
     (   catch(relative_file(Module:Head, File), _, fail)
     ->  uri_data(path, Components, File),
         uri_components(HREF, Components)
-    ;   in_file(Module:Head, File0),
-        insert_alias(File0, File),
+    ;   in_file(Module:Head, File0)
+    ->  insert_alias(File0, File),
         http_location_by_id(pldoc_doc, DocHandler),
         atom_concat(DocHandler, File, Path),
         uri_data(path, Components, Path),
@@ -2140,19 +2166,27 @@ in_file(Head, File) :-
     distinct(File, in_file(_, Head, File)).
 
 in_file(Module, Head, File) :-
+    var(Module),
+    (   predicate_property(system:Head, foreign)
+    ->  !,
+        fail
+    ;   predicate_property(system:Head, file(File)),
+        \+ system_arithmetic_function(Head)
+    ->  !
+    ;   predicate_property(Head, autoload(File0))
+    ->  !,
+        file_name_extension(File0, pl, File)
+    ;   exported_from(Module, Head, File),
+        module_property(Module, class(library))
+    ).
+in_file(Module, Head, File) :-
+    xref_defined(File, Head, How),
     xref_current_source(File),
     atom(File),                     % only plain files
     xref_module(File, Module),
-    xref_defined(File, Head, How),
     How \= imported(_From).
 in_file(Module, Head, File) :-
-    distinct(Primary,
-             (predicate_property(Module:Head, exported),
-              (   predicate_property(Module:Head, imported_from(Primary))
-              ->  true
-              ;   Primary = Module
-              ))),
-    module_property(Primary, file(File)).
+    exported_from(Module, Head, File).
 in_file(Module, Head, File) :-
     predicate_property(Module:Head, file(File)),
     \+ predicate_property(Module:Head, imported_from(_)).
@@ -2160,6 +2194,25 @@ in_file(Module, Head, File) :-
     current_module(Module),
     source_file(Module:Head, File).
 
+exported_from(Module, Head, File) :-
+    distinct(Primary,
+             (   predicate_property(Module:Head, exported),
+                 (   predicate_property(Module:Head, imported_from(Primary))
+                 ->  true
+                 ;   Primary = Module
+                 ))),
+    module_property(Primary, file(File)).
+
+:- multifile
+    arithmetic:evaluable/2.
+
+system_arithmetic_function(Head) :-
+    functor(Head, Name, Arity),
+    FArith is Arity-1,
+    FArith >= 0,
+    functor(FHead, Name, FArith),
+    arithmetic:evaluable(FHead, system).
+
 %%     file(+FileName)// is det.
 %%     file(+FileName, +Options)// is det.
 %
