@@ -68,7 +68,7 @@ extern "C" {
 /* PLVERSION_TAG: a string, normally "", but for example "rc1" */
 
 #ifndef PLVERSION
-#define PLVERSION 80121
+#define PLVERSION 80122
 #endif
 #ifndef PLVERSION_TAG
 #define PLVERSION_TAG ""
@@ -204,34 +204,35 @@ typedef union
 #define	PL_VARIABLE	 (1)		/* nothing */
 #define PL_ATOM		 (2)		/* const char * */
 #define PL_INTEGER	 (3)		/* int */
-#define PL_FLOAT	 (4)		/* double */
-#define PL_STRING	 (5)		/* const char * */
-#define PL_TERM		 (6)
+#define PL_RATIONAL	 (4)		/* rational number */
+#define PL_FLOAT	 (5)		/* double */
+#define PL_STRING	 (6)		/* const char * */
+#define PL_TERM		 (7)
 
-#define PL_NIL		 (7)		/* The constant [] */
-#define PL_BLOB		 (8)		/* non-atom blob */
-#define PL_LIST_PAIR	 (9)		/* [_|_] term */
+#define PL_NIL		 (8)		/* The constant [] */
+#define PL_BLOB		 (9)		/* non-atom blob */
+#define PL_LIST_PAIR	 (10)		/* [_|_] term */
 
 					/* PL_unify_term() */
-#define PL_FUNCTOR	 (10)		/* functor_t, arg ... */
-#define PL_LIST		 (11)		/* length, arg ... */
-#define PL_CHARS	 (12)		/* const char * */
-#define PL_POINTER	 (13)		/* void * */
+#define PL_FUNCTOR	 (11)		/* functor_t, arg ... */
+#define PL_LIST		 (12)		/* length, arg ... */
+#define PL_CHARS	 (13)		/* const char * */
+#define PL_POINTER	 (14)		/* void * */
 					/* PlArg::PlArg(text, type) */
-#define PL_CODE_LIST	 (14)		/* [ascii...] */
-#define PL_CHAR_LIST	 (15)		/* [h,e,l,l,o] */
-#define PL_BOOL		 (16)		/* PL_set_prolog_flag() */
-#define PL_FUNCTOR_CHARS (17)		/* PL_unify_term() */
-#define _PL_PREDICATE_INDICATOR (18)	/* predicate_t (Procedure) */
-#define PL_SHORT	 (19)		/* short */
-#define PL_INT		 (20)		/* int */
-#define PL_LONG		 (21)		/* long */
-#define PL_DOUBLE	 (22)		/* double */
-#define PL_NCHARS	 (23)		/* size_t, const char * */
-#define PL_UTF8_CHARS	 (24)		/* const char * */
-#define PL_UTF8_STRING	 (25)		/* const char * */
-#define PL_INT64	 (26)		/* int64_t */
-#define PL_NUTF8_CHARS	 (27)		/* size_t, const char * */
+#define PL_CODE_LIST	 (15)		/* [ascii...] */
+#define PL_CHAR_LIST	 (16)		/* [h,e,l,l,o] */
+#define PL_BOOL		 (17)		/* PL_set_prolog_flag() */
+#define PL_FUNCTOR_CHARS (18)		/* PL_unify_term() */
+#define _PL_PREDICATE_INDICATOR (19)	/* predicate_t (Procedure) */
+#define PL_SHORT	 (20)		/* short */
+#define PL_INT		 (21)		/* int */
+#define PL_LONG		 (22)		/* long */
+#define PL_DOUBLE	 (23)		/* double */
+#define PL_NCHARS	 (24)		/* size_t, const char * */
+#define PL_UTF8_CHARS	 (25)		/* const char * */
+#define PL_UTF8_STRING	 (26)		/* const char * */
+#define PL_INT64	 (27)		/* int64_t */
+#define PL_NUTF8_CHARS	 (28)		/* size_t, const char * */
 #define PL_NUTF8_CODES	 (29)		/* size_t, const char * */
 #define PL_NUTF8_STRING	 (30)		/* size_t, const char * */
 #define PL_NWCHARS	 (31)		/* size_t, const wchar_t * */
@@ -808,27 +809,29 @@ PL_EXPORT(int)		_PL_get_arg(int index, term_t t, term_t a);
 		 *	    CHAR BUFFERS	*
 		 *******************************/
 
-#define CVT_ATOM	0x0001
-#define CVT_STRING	0x0002
-#define CVT_LIST	0x0004
-#define CVT_INTEGER	0x0008
-#define CVT_FLOAT	0x0010
-#define CVT_VARIABLE	0x0020
-#define CVT_NUMBER	(CVT_INTEGER|CVT_FLOAT)
-#define CVT_ATOMIC	(CVT_NUMBER|CVT_ATOM|CVT_STRING)
-#define CVT_WRITE	0x0040
-#define CVT_WRITE_CANONICAL 0x0080
-#define CVT_WRITEQ	0x00C0
-#define CVT_ALL		(CVT_ATOMIC|CVT_LIST)
-#define CVT_MASK	0x00ff
+#define CVT_ATOM	    0x00000001
+#define CVT_STRING	    0x00000002
+#define CVT_LIST	    0x00000004
+#define CVT_INTEGER	    0x00000008
+#define CVT_RATIONAL	    0x00000010
+#define CVT_FLOAT	    0x00000020
+#define CVT_VARIABLE	    0x00000040
+#define CVT_NUMBER	    (CVT_RATIONAL|CVT_FLOAT)
+#define CVT_ATOMIC	    (CVT_NUMBER|CVT_ATOM|CVT_STRING)
+#define CVT_WRITE	    0x00000080
+#define CVT_WRITE_CANONICAL 0x00000080
+#define CVT_WRITEQ	    0x000000C0
+#define CVT_ALL		    (CVT_ATOMIC|CVT_LIST)
+#define CVT_MASK	    0x00000fff
 
-#define BUF_DISCARDABLE	0x0000
-#define BUF_RING	0x0100
-#define BUF_MALLOC	0x0200
-#define BUF_ALLOW_STACK	0x0400		/* allow pointer into (global) stack */
+#define CVT_EXCEPTION	    0x00001000	/* throw exception on error */
+#define CVT_VARNOFAIL	    0x00002000	/* return 2 if argument is unbound */
+
+#define BUF_DISCARDABLE	    0x00000000
+#define BUF_RING	    0x00010000
+#define BUF_MALLOC	    0x00020000
+#define BUF_ALLOW_STACK	    0x00040000	/* allow pointer into (global) stack */
 
-#define CVT_EXCEPTION	0x10000		/* throw exception on error */
-#define CVT_VARNOFAIL	0x20000		/* return 2 if argument is unbound */
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Output   representation   for   PL_get_chars()     and    friends.   The
@@ -837,16 +840,16 @@ Windows we use UTF-8 which is translated   by the `XOS' layer to Windows
 UNICODE file functions.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-#define REP_ISO_LATIN_1 0x0000		/* output representation */
-#define REP_UTF8	0x1000
-#define REP_MB		0x2000
+#define REP_ISO_LATIN_1	    0x00000000	/* output representation */
+#define REP_UTF8	    0x00100000
+#define REP_MB		    0x00200000
 #ifdef __WINDOWS__
-#define REP_FN		REP_UTF8
+#define REP_FN		    REP_UTF8
 #else
-#define REP_FN		REP_MB
+#define REP_FN		    REP_MB
 #endif
 
-#define PL_DIFF_LIST	0x20000		/* PL_unify_chars() */
+#define PL_DIFF_LIST	    0x01000000	/* PL_unify_chars() */
 
 
 #ifdef SIO_MAGIC			/* defined from <SWI-Stream.h> */
@@ -902,6 +905,7 @@ PL_EXPORT(IOSTREAM *)*_PL_streams(void);	/* base of streams */
 #define PL_WRT_NODICT	       0x40000	/* Do not write dicts in pretty syntax */
 #define PL_WRT_NODOTINATOM     0x80000	/* never write a.b unquoted */
 #define PL_WRT_NO_LISTS	       0x100000	/* Do not write lists as [...] */
+#define PL_WRT_RAT_NATURAL     0x200000	/* Write rationals as 1/3 */
 
 PL_EXPORT(int)	PL_write_term(IOSTREAM *s,
 			     term_t term,
