@@ -183,7 +183,8 @@ PL_get_text__LD(term_t l, PL_chars_t *text, int flags ARG_LD)
       goto maybe_write;
     if ( !PL_from_stack_text(text, flags) )
       return FALSE;			/* no memory */
-  } else if ( (flags & CVT_INTEGER) && isInteger(w) )
+  } else if ( ((flags&CVT_RATIONAL) && isRational(w)) ||
+	      ((flags&CVT_INTEGER)  && isInteger(w)) )
   { number n;
 
     PL_get_number(l, &n);
@@ -209,6 +210,24 @@ PL_get_text__LD(term_t l, PL_chars_t *text, int flags ARG_LD)
 	text->length  = entriesBuffer(b, char);
 	text->storage = PL_CHARS_RING;
 
+	break;
+      }
+      case V_MPQ:
+      { size_t sz = ( mpz_sizeinbase(mpq_numref(n.value.mpq), 10) +
+		      mpz_sizeinbase(mpq_denref(n.value.mpq), 10) + 4 );
+	Buffer b  = findBuffer(BUF_RING);
+
+	if ( !growBuffer(b, sz) )
+	  outOfCore();
+	mpz_get_str(b->base, 10, mpq_numref(n.value.mpq));
+	b->top = b->base + strlen(b->base);
+	*b->top = '/';
+	mpz_get_str(b->top, 10, mpq_denref(n.value.mpq));
+	b->top += strlen(b->top);
+	text->text.t  = baseBuffer(b, char);
+	text->length  = entriesBuffer(b, char);
+	text->storage = PL_CHARS_RING;
+
 	break;
       }
 #endif
@@ -397,7 +416,9 @@ textToString(PL_chars_t *text)
 
 int
 PL_unify_text(term_t term, term_t tail, PL_chars_t *text, int type)
-{ switch(type)
+{ GET_LD
+
+  switch(type)
   { case PL_ATOM:
     { atom_t a = textToAtom(text);
 
@@ -426,15 +447,13 @@ PL_unify_text(term_t term, term_t tail, PL_chars_t *text, int type)
 
       if ( text->length == 0 )
       { if ( tail )
-	{ GET_LD
-	  PL_put_term(tail, term);
+	{ PL_put_term(tail, term);
 	  return TRUE;
 	} else
 	{ return PL_unify_nil(term);
 	}
       } else
-      { GET_LD
-	term_t l = PL_new_term_ref();
+      { term_t l = PL_new_term_ref();
 	Word p0, p;
 
 	switch(text->encoding)
