@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2017, University of Amsterdam
+    Copyright (c)  1985-2020, University of Amsterdam
                               VU University Amsterdam
+			      CWI, Amsterdam
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -69,6 +70,7 @@ in this array.
 #ifdef HAVE_IEEEFP_H
 #include <ieeefp.h>
 #endif
+#include <fenv.h>
 
 #ifdef fpclassify
 #define HAVE_FPCLASSIFY 1
@@ -84,6 +86,21 @@ in this array.
 #define M_E (2.7182818284590452354)
 #endif
 
+/* LD->arith.float_flags values */
+#define FLT_ROUND_NEAREST	0x0001
+#define FLT_ROUND_TO_POS	0x0002
+#define FLT_ROUND_TO_NEG	0x0003
+#define FLT_ROUND_TO_ZERO	0x0004
+#define FLT_ROUND_MASK		0x000f
+#define FLT_OVERFLOW		0x0010
+#define FLT_ZERO_DIV		0x0020
+#define FLT_UNDEFINED		0x0040
+#define FLT_UNDERFLOW		0x0080
+
+static double const_nan;
+static double const_inf;
+static double const_neg_inf;
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 On some machines, notably  FreeBSD  upto   version  3.x,  floating point
 operations raise signals rather then leaving an error condition and this
@@ -444,6 +461,45 @@ out:
   return rc;
 }
 
+
+static
+PRED_IMPL("rational", 3, rational, 0)
+{ PRED_LD
+  Word p = valTermRef(A1);
+
+  deRef(p);
+  if ( isRational(*p) )
+  { if ( isMPQNum(*p) )
+    { number n, num, den;
+      int rc;
+
+      get_rational(*p, &n);
+      assert(n.type == V_MPQ);
+
+      num.type = V_MPZ;
+      den.type = V_MPZ;
+      mpz_init(num.value.mpz);
+      mpz_init(den.value.mpz);
+      mpz_set(num.value.mpz, mpq_numref(n.value.mpq));
+      mpz_set(den.value.mpz, mpq_denref(n.value.mpq));
+
+      rc = ( PL_unify_number(A2, &num) &&
+	     PL_unify_number(A3, &den) );
+
+      clearNumber(&num);
+      clearNumber(&den);
+
+      return rc;
+    } else
+    { return ( PL_unify(A1, A2) &&
+	       PL_unify_integer(A3, 1) );
+    }
+  }
+
+  return FALSE;
+}
+
+
 #endif /*O_GMP*/
 
 		/********************************
@@ -602,14 +658,17 @@ isCurrentArithFunction(functor_t f)
 
 int
 check_float(double f)
-{
+{ PL_error_code code = ERR_NO_ERROR;
 #ifdef HAVE_FPCLASSIFY
   switch(fpclassify(f))
   { case FP_NAN:
-      return PL_error(NULL, 0, NULL, ERR_AR_UNDEF);
+      code = ERR_AR_UNDEF;
+      break;
+    case FP_SUBNORMAL:
+      code = ERR_AR_UNDERFLOW;
       break;
     case FP_INFINITE:
-      return PL_error(NULL, 0, NULL, ERR_AR_OVERFLOW);
+      code = ERR_AR_OVERFLOW;
       break;
   }
 #else
@@ -617,14 +676,16 @@ check_float(double f)
   switch(fpclass(f))
   { case FP_SNAN:
     case FP_QNAN:
-      return PL_error(NULL, 0, NULL, ERR_AR_UNDEF);
+      code = ERR_AR_UNDEF;
       break;
     case FP_NINF:
     case FP_PINF:
-      return PL_error(NULL, 0, NULL, ERR_AR_OVERFLOW);
+      code = ERR_AR_OVERFLOW;
       break;
     case FP_NDENORM:			/* pos/neg denormalized non-zero */
     case FP_PDENORM:
+      code = ERR_AR_UNDERFLOW;
+      break;
     case FP_NNORM:			/* pos/neg normalized non-zero */
     case FP_PNORM:
     case FP_NZERO:			/* pos/neg zero */
@@ -636,29 +697,78 @@ check_float(double f)
   switch(_fpclass(f))
   { case _FPCLASS_SNAN:
     case _FPCLASS_QNAN:
-      return PL_error(NULL, 0, NULL, ERR_AR_UNDEF);
+      code = ERR_AR_UNDEF;
       break;
     case _FPCLASS_NINF:
     case _FPCLASS_PINF:
-      return PL_error(NULL, 0, NULL, ERR_AR_OVERFLOW);
+      code = ERR_AR_OVERFLOW;
       break;
   }
 #else
 #ifdef HAVE_ISNAN
   if ( isnan(f) )
-    return PL_error(NULL, 0, NULL, ERR_AR_UNDEF);
+    code = ERR_AR_UNDEF;
 #endif
 #ifdef HAVE_ISINF
   if ( isinf(f) )
-    return PL_error(NULL, 0, NULL, ERR_AR_OVERFLOW);
+    code = ERR_AR_OVERFLOW;
 #endif
 #endif /*HAVE__FPCLASS*/
 #endif /*HAVE_FPCLASS*/
 #endif /*HAVE_FPCLASSIFY*/
+
+  if ( code != ERR_NO_ERROR )
+  { GET_LD
+
+    switch(code)
+    { case ERR_AR_OVERFLOW:
+	if ( LD->arith.f.flags & FLT_OVERFLOW )
+	  return TRUE;
+        break;
+      case ERR_AR_UNDERFLOW:
+	if ( LD->arith.f.flags & FLT_UNDERFLOW )
+	  return TRUE;
+        break;
+      case ERR_AR_UNDEF:
+	if ( LD->arith.f.flags & FLT_UNDEFINED )
+	  return TRUE;
+        break;
+      default:
+	assert(0);
+    }
+
+    return PL_error(NULL, 0, NULL, code);
+  }
+
   return TRUE;
 }
 
 
+#ifdef O_GMP
+static int
+check_mpq(Number r)
+{ GET_LD
+  size_t sz;
+
+  if ( (sz=LD->arith.rat.max_rational_size) != (size_t)-1 )
+  { if ( ( mpq_numref(r->value.mpq)->_mp_size +
+	   mpq_denref(r->value.mpq)->_mp_size ) * sizeof(mp_limb_t) > sz )
+    { atom_t action = LD->arith.rat.max_rational_size_action;
+
+      if ( action == ATOM_float )
+	promoteToFloatNumber(r);
+      else if ( action == ATOM_error )
+	return PL_error(NULL, 0, "requires more than max_rational_size bytes",
+			ERR_AR_TRIPWIRE, ATOM_max_rational_size, r);
+      else
+	assert(0);
+    }
+  }
+
+  return TRUE;
+}
+#endif
+
 		 /*******************************
 		 *	     EVALULATE		*
 		 *******************************/
@@ -716,7 +826,7 @@ valueExpression(term_t expr, number *result ARG_LD)
   for(;;)
   { switch(tag(*p))
     { case TAG_INTEGER:
-	get_integer(*p, n);
+	get_rational(*p, n);
         break;
       case TAG_FLOAT:
 	n->value.f = valFloat(*p);
@@ -1154,7 +1264,7 @@ ar_add_ui(Number n, intptr_t add)
 	mpz_submul_ui(mpq_numref(n->value.mpq), mpq_denref(n->value.mpq),
 		      (unsigned long)-add);
 
-      succeed;
+      return check_mpq(n);
     }
 #endif
     case V_FLOAT:
@@ -1208,7 +1318,7 @@ pl_ar_add(Number n1, Number n2, Number r)
     { r->type = V_MPQ;
       mpq_init(r->value.mpq);
       mpq_add(r->value.mpq, n1->value.mpq, n2->value.mpq);
-      succeed;
+      return check_mpq(r);
     }
 #endif
     case V_FLOAT:
@@ -1255,6 +1365,7 @@ ar_minus(Number n1, Number n2, Number r)
     { r->type = V_MPQ;
       mpq_init(r->value.mpq);
       mpq_sub(r->value.mpq, n1->value.mpq, n2->value.mpq);
+      return check_mpq(r);
       succeed;
     }
 #endif
@@ -1696,11 +1807,11 @@ static int
 ar_pow(Number n1, Number n2, Number r)
 {
 #ifdef O_GMP
-  if ( intNumber(n1) && intNumber(n2) )
-  { unsigned long exp;
-    int exp_sign;
+  unsigned long exp;
+  int exp_sign;
 
-    switch(n1->type)			/* test for 0**X and 1**X */
+  if ( intNumber(n1) && intNumber(n2) )
+  { switch(n1->type)			/* test for 0**X and 1**X */
     { case V_INTEGER:
 	if ( n1->value.i == 0 )
 	{ ret0:
@@ -1742,7 +1853,14 @@ ar_pow(Number n1, Number n2, Number r)
     if ( !get_int_exponent(n2, &exp, &exp_sign) )
       return FALSE;
     if ( exp_sign < 0 )
+    { GET_LD
+
+      if ( truePrologFlag(PLFLAG_RATIONAL) )
+      { promoteToMPQNumber(n1);
+	goto int_pow_neg_int;
+      }
       goto doreal;
+    }
     if ( exp_sign == 0 )
     { r->type = V_INTEGER;
       r->value.i = 1;
@@ -1790,12 +1908,10 @@ ar_pow(Number n1, Number n2, Number r)
 	assert(0);
         fail;
     }
-  }
+  } /* end if ( intNumber(n1) && intNumber(n2) ) */
 
   if ( n1->type == V_MPQ && intNumber(n2) )
   { number nr, nd, nrp, ndp, nexp;
-    unsigned long exp;
-    int exp_sign;
 
     if ( !get_int_exponent(n2, &exp, &exp_sign) )
       return FALSE;
@@ -1806,6 +1922,7 @@ ar_pow(Number n1, Number n2, Number r)
       return TRUE;
     }
 
+  int_pow_neg_int:
     nexp.type = V_INTEGER;
     nexp.value.i = exp;
 
@@ -1834,12 +1951,127 @@ ar_pow(Number n1, Number n2, Number r)
       clearNumber(&nrp);
       clearNumber(&ndp);
 
-      return TRUE;
+      return check_mpq(r);
     }
 
     clearNumber(&nrp);
     clearNumber(&ndp);
-  }
+  } /* end MPQ^int */
+
+  if ( n2->type == V_MPQ )			/* X ^ rat */
+  { int sgn_exp = mpq_sgn(n2->value.mpq);
+    long r_den;
+
+    if ( sgn_exp == -1 )
+      mpz_neg(mpq_numref(n2->value.mpq), mpq_numref(n2->value.mpq));
+
+    r_den = mpz_get_ui(mpq_denref(n2->value.mpq));
+
+    switch (n1->type)
+    { case V_INTEGER:
+      case V_MPZ:				/* int ^ rat */
+      { r->type = V_MPZ;
+
+	if ( n1->type == V_INTEGER )
+	  mpz_init_set_si(r->value.mpz,n1->value.i);
+	else
+	  mpz_init_set(r->value.mpz,n1->value.mpz);
+
+	/* neg ^ int/even is undefined */
+	if ( mpz_sgn(r->value.mpz) == -1 && !(r_den & 1))
+	{ mpz_clear(r->value.mpz);
+	  r->type = V_FLOAT;
+	  r->value.f = NAN;
+	  return check_float(r->value.f);
+	}
+
+	if ( mpz_root(r->value.mpz,r->value.mpz,r_den))
+	{ unsigned long r_num = mpz_get_ui(mpq_numref(n2->value.mpq));
+
+	  if ( r_num > LONG_MAX )
+	  { mpz_clear(r->value.mpz);
+	    goto doreal;
+	  } else
+	  { mpz_pow_ui(r->value.mpz,r->value.mpz,r_num);
+
+	    if (sgn_exp == -1)  /* create mpq=1/r->value */
+	    { mpz_t tempz;
+
+	      mpz_init_set(tempz,r->value.mpz);
+	      mpz_clear(r->value.mpz);
+	      r->type = V_MPQ;
+	      mpq_init(r->value.mpq);
+	      mpq_set_z(r->value.mpq,tempz);
+	      mpq_inv(r->value.mpq,r->value.mpq);
+	      mpz_clear(tempz);
+	      return check_mpq(r);
+	    } else
+	    { return TRUE;
+	    }
+	  }
+	} else
+	{ mpz_clear(r->value.mpz);
+	  goto doreal;
+	}
+	break;
+      }
+      case V_MPQ:
+      { int rat_result;
+	unsigned long r_num;
+
+	r->type = V_MPQ;
+	mpq_init(r->value.mpq);
+	mpq_set(r->value.mpq, n1->value.mpq);
+
+	/* neg ^ int / even */
+	if ( (mpq_sgn(r->value.mpq) == -1 ) && !(r_den & 1))
+	{ mpq_clear(r->value.mpq);
+	  r->type = V_FLOAT;
+	  r->value.f = NAN;
+	  return check_float(r->value.f);
+	}
+
+	rat_result = ( mpz_root(mpq_numref(r->value.mpq),
+				mpq_numref(r->value.mpq),r_den) &&
+		       mpz_root(mpq_denref(r->value.mpq),
+				mpq_denref(r->value.mpq),r_den)
+		     );
+
+	r_num = mpz_get_ui(mpq_numref(n2->value.mpq));
+	if ( rat_result && (r_num < LONG_MAX) )	/* base = base^P */
+	{ mpz_pow_ui(mpq_numref(r->value.mpq),mpq_numref(r->value.mpq),r_num);
+	  mpz_pow_ui(mpq_denref(r->value.mpq),mpq_denref(r->value.mpq),r_num);
+
+	  /* if original exponent negative, invert */
+	  if ( sgn_exp == -1 )
+	    mpq_inv(r->value.mpq,r->value.mpq);
+
+	  return check_mpq(r);
+	} else
+	{ mpq_clear(r->value.mpq);
+	  goto doreal;
+	}
+	assert(0);
+      }
+      case V_FLOAT:
+      { double d_exp = sgn_exp*mpq_get_d(n2->value.mpq);
+
+	r->type = V_FLOAT;
+	if ( n1->value.f < 0 )
+	{ if ( r_den & 1 )			/* odd denominator */
+	  { int sign = mpz_divisible_2exp_p(mpq_numref(n2->value.mpq),1) ? 1 : -1;
+	    r->value.f = sign * pow(-(n1->value.f),d_exp);
+	  } else
+	  { r->value.f = NAN;			/* even denominator */
+	  }
+	} else					/* base positive */
+	{ r->value.f = pow(n1->value.f,d_exp);
+	}
+	return check_float(r->value.f);
+      }
+    } /* end switch (n1->type) */
+    assert(0);
+  } /* end if ( n2->type == V_MPQ ) */
 
 doreal:
 #endif /*O_GMP*/
@@ -1883,13 +2115,36 @@ ar_powm(Number base, Number exp, Number mod, Number r)
 #endif
 }
 
+#define AR_UNDEFINED_IF(func, arity, test, r)			\
+	if ( test )						\
+	{ GET_LD						\
+	  if ( LD->arith.f.flags & FLT_UNDEFINED )		\
+	  { r->type = V_FLOAT;					\
+	    r->value.f = const_nan;				\
+	    return TRUE;					\
+	  } else						\
+	  { return PL_error(func, arity, NULL, ERR_AR_UNDEF);	\
+	  }							\
+	}
+#define AR_DIV_ZERO_IF(func, arity, n, d, r)			\
+	if ( d == 0.0 )						\
+	{ GET_LD						\
+	  if ( LD->arith.f.flags & FLT_ZERO_DIV )		\
+	  { r->type = V_FLOAT;					\
+	    r->value.f = signbit(n) == signbit(d)		\
+			? const_inf				\
+			: const_neg_inf;			\
+	    return TRUE;					\
+	  } else						\
+	  { return PL_error(func, arity, NULL, ERR_DIV_BY_ZERO);\
+	  }							\
+	}
 
 static int
 ar_sqrt(Number n1, Number r)
 { if ( !promoteToFloatNumber(n1) )
     return FALSE;
-  if ( n1->value.f < 0 )
-    return PL_error("sqrt", 1, NULL, ERR_AR_UNDEF);
+  AR_UNDEFINED_IF("sqrt", 1,  n1->value.f < 0, r);
   r->value.f = sqrt(n1->value.f);
   r->type    = V_FLOAT;
 
@@ -1901,8 +2156,7 @@ static int
 ar_asin(Number n1, Number r)
 { if ( !promoteToFloatNumber(n1) )
     return FALSE;
-  if ( n1->value.f < -1.0 || n1->value.f > 1.0 )
-    return PL_error("asin", 1, NULL, ERR_AR_UNDEF);
+  AR_UNDEFINED_IF("asin", 1, n1->value.f < -1.0 || n1->value.f > 1.0, r);
   r->value.f = asin(n1->value.f);
   r->type    = V_FLOAT;
 
@@ -1914,8 +2168,7 @@ static int
 ar_acos(Number n1, Number r)
 { if ( !promoteToFloatNumber(n1) )
     return FALSE;
-  if ( n1->value.f < -1.0 || n1->value.f > 1.0 )
-    return PL_error("acos", 1, NULL, ERR_AR_UNDEF);
+  AR_UNDEFINED_IF("ascos", 1, n1->value.f < -1.0 || n1->value.f > 1.0, r);
   r->value.f = acos(n1->value.f);
   r->type    = V_FLOAT;
 
@@ -1927,8 +2180,7 @@ static int
 ar_log(Number n1, Number r)
 { if ( !promoteToFloatNumber(n1) )
     return FALSE;
-  if ( n1->value.f <= 0.0 )
-    return PL_error("log", 1, NULL, ERR_AR_UNDEF);
+  AR_UNDEFINED_IF("log", 1, n1->value.f <= 0.0 , r);
   r->value.f = log(n1->value.f);
   r->type    = V_FLOAT;
 
@@ -1940,8 +2192,7 @@ static int
 ar_log10(Number n1, Number r)
 { if ( !promoteToFloatNumber(n1) )
     return FALSE;
-  if ( n1->value.f <= 0.0 )
-    return PL_error("log10", 1, NULL, ERR_AR_UNDEF);
+  AR_UNDEFINED_IF("log10", 1, n1->value.f <= 0.0, r);
   r->value.f = log10(n1->value.f);
   r->type    = V_FLOAT;
 
@@ -2173,6 +2424,22 @@ ar_copysign(Number n1, Number n2, Number r)
 }
 
 
+static int
+ar_nexttoward(Number n1, Number n2, Number r)
+{ if ( promoteToFloatNumber(n1) &&
+       promoteToFloatNumber(n2) )
+  { if ( n1->type == V_FLOAT && n2->type == V_FLOAT )
+    { r->value.f = nexttoward(n1->value.f, n2->value.f);
+      r->type = V_FLOAT;
+
+      return TRUE;
+    }
+  }
+
+  return FALSE;
+}
+
+
 static int
 ar_rem(Number n1, Number n2, Number r)
 { if ( !toIntegerNumber(n1, 0) )
@@ -2224,6 +2491,41 @@ ar_rational(Number n1, Number r)
   succeed;
 }
 
+static int
+ar_numerator(Number n1, Number r)
+{ if ( intNumber(n1) )
+  { cpNumber(r, n1);
+    return TRUE;
+  }
+  if ( n1->type == V_MPQ )
+  { r->type = V_MPZ;
+    mpz_init(r->value.mpz);
+    mpz_set(r->value.mpz, mpq_numref(n1->value.mpq));
+    return TRUE;
+  }
+
+  return PL_error("numerator", 1, NULL, ERR_AR_TYPE, ATOM_rational, n1);
+}
+
+
+static int
+ar_denominator(Number n1, Number r)
+{ if ( intNumber(n1) )
+  { r->type = V_INTEGER;
+    r->value.i = 1;
+    return TRUE;
+  }
+  if ( n1->type == V_MPQ )
+  { r->type = V_MPZ;
+    mpz_init(r->value.mpz);
+    mpz_set(r->value.mpz, mpq_denref(n1->value.mpq));
+    return TRUE;
+  }
+
+  return PL_error("denominator", 1, NULL, ERR_AR_TYPE, ATOM_rational, n1);
+}
+
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 A is rationalize(Float)
 
@@ -2263,7 +2565,7 @@ ar_rationalize(Number n1, Number r)
     case V_MPQ:
       cpNumber(r, n1);
       promoteToMPQNumber(r);
-      succeed;
+      return check_mpq(r);
     case V_FLOAT:
     { if ( !check_float(n1->value.f) )
 	return FALSE;
@@ -2295,8 +2597,7 @@ ar_rationalize(Number n1, Number r)
       mpz_init_set_d(mpq_numref(r->value.mpq), p1);
       mpz_init_set_d(mpq_denref(r->value.mpq), q1);
       mpq_canonicalize(r->value.mpq);	/* is this needed? */
-
-      succeed;
+      return check_mpq(r);
     }
   }
 
@@ -2305,6 +2606,25 @@ ar_rationalize(Number n1, Number r)
 }
 
 
+int
+ar_rdiv_mpz(Number n1, Number n2, Number r)
+{ if ( mpz_divisible_p(n1->value.mpz, n2->value.mpz) )
+  { mpz_init(r->value.mpz);
+    r->type = V_MPZ;
+    mpz_divexact(r->value.mpz, n1->value.mpz, n2->value.mpz);
+  } else
+  { r->type = V_MPQ;
+    mpq_init(r->value.mpq);
+    mpz_set(mpq_numref(r->value.mpq), n1->value.mpz);
+    mpz_set(mpq_denref(r->value.mpq), n2->value.mpz);
+    mpq_canonicalize(r->value.mpq);
+    return check_mpq(r);
+  }
+
+  return TRUE;
+}
+
+
 static int
 ar_rdiv(Number n1, Number n2, Number r)
 { if ( toIntegerNumber(n1, 0) &&
@@ -2314,18 +2634,9 @@ ar_rdiv(Number n1, Number n2, Number r)
 
     if ( mpz_sgn(n2->value.mpz) == 0 )
       return PL_error("/", 2, NULL, ERR_DIV_BY_ZERO);
-    if ( mpz_divisible_p(n1->value.mpz, n2->value.mpz) )
-    { mpz_init(r->value.mpz);
-      r->type = V_MPZ;
-      mpz_divexact(r->value.mpz, n1->value.mpz, n2->value.mpz);
-      succeed;
-    }
-    r->type = V_MPQ;
-    mpq_init(r->value.mpq);
-    mpz_set(mpq_numref(r->value.mpq), n1->value.mpz);
-    mpz_set(mpq_denref(r->value.mpq), n2->value.mpz);
-    mpq_canonicalize(r->value.mpq);
-  } else
+
+    return ar_rdiv_mpz(n1, n2, r);
+  } else if ( ratNumber(n1) && ratNumber(n2) )
   { promoteToMPQNumber(n1);
     promoteToMPQNumber(n2);
 
@@ -2335,9 +2646,14 @@ ar_rdiv(Number n1, Number n2, Number r)
     r->type = V_MPQ;
     mpq_init(r->value.mpq);
     mpq_div(r->value.mpq, n1->value.mpq, n2->value.mpq);
+    return check_mpq(r);
+  } else if ( !ratNumber(n1) )
+  { return PL_error("rdiv", 2, NULL, ERR_AR_TYPE, ATOM_rational, n1);
+  } else
+  { return PL_error("rdiv", 2, NULL, ERR_AR_TYPE, ATOM_rational, n2);
   }
 
-  succeed;
+  return TRUE;
 }
 #endif /*O_GMP*/
 
@@ -2359,6 +2675,13 @@ ar_divide(Number n1, Number n2, Number r)
 	  r->type = V_INTEGER;
 	  succeed;
 	}
+#ifdef O_GMP
+	if ( truePrologFlag(PLFLAG_RATIONAL) )
+	{ promoteToMPZNumber(n1);
+	  promoteToMPZNumber(n2);
+	  return ar_rdiv_mpz(n1, n2, r);
+	}
+#endif
 	break;
 #ifdef O_GMP
       case V_MPZ:
@@ -2370,6 +2693,8 @@ ar_divide(Number n1, Number n2, Number r)
 	  mpz_divexact(r->value.mpz, n1->value.mpz, n2->value.mpz);
 	  succeed;
 	}
+	if ( truePrologFlag(PLFLAG_RATIONAL) )
+	  return ar_rdiv_mpz(n1, n2, r);
         break;
       case V_MPQ:
 	if ( mpq_sgn(n2->value.mpq) == 0 )
@@ -2377,7 +2702,7 @@ ar_divide(Number n1, Number n2, Number r)
         mpq_init(r->value.mpq);
 	r->type = V_MPQ;
 	mpq_div(r->value.mpq, n1->value.mpq, n2->value.mpq);
-	succeed;
+	return check_mpq(r);
 #endif
       case V_FLOAT:
 	break;
@@ -2388,8 +2713,7 @@ ar_divide(Number n1, Number n2, Number r)
   if ( !promoteToFloatNumber(n1) ||
        !promoteToFloatNumber(n2) )
     return FALSE;
-  if ( n2->value.f == 0.0 )
-    return PL_error("/", 2, NULL, ERR_DIV_BY_ZERO);
+  AR_DIV_ZERO_IF("/", 2, n1->value.f, n2->value.f, r);
   r->value.f = n1->value.f / n2->value.f;
   r->type = V_FLOAT;
 
@@ -2497,7 +2821,7 @@ ar_mul(Number n1, Number n2, Number r)
       r->type = V_MPQ;
       mpq_init(r->value.mpq);
       mpq_mul(r->value.mpq, n1->value.mpq, n2->value.mpq);
-      succeed;
+      return check_mpq(r);
 #else
       return PL_error("*", 2, NULL, ERR_EVALUATION, ATOM_int_overflow);
 #endif
@@ -2802,7 +3126,7 @@ ar_u_minus(Number n1, Number r)
     case V_MPQ:
       mpq_init(r->value.mpq);
       mpq_neg(r->value.mpq, n1->value.mpq);
-      break;
+      return check_mpq(r);
 #endif
     case V_FLOAT:
       r->value.f = -n1->value.f;
@@ -3507,7 +3831,7 @@ ar_inf(Number r)
 { static number n = {0};
 
   if ( n.type != V_FLOAT )
-  { n.value.f = strtod("Inf", NULL);
+  { n.value.f = const_inf;
     n.type = V_FLOAT;
   }
 
@@ -3522,7 +3846,7 @@ ar_nan(Number r)
 { static number n = {0};
 
   if ( n.type != V_FLOAT )
-  { n.value.f = strtod("NaN", NULL);
+  { n.value.f = const_nan;
     n.type = V_FLOAT;
   }
 
@@ -3641,6 +3965,8 @@ static const ar_funcdef ar_funcdefs[] = {
   ADD(FUNCTOR_divide2,		ar_divide, F_ISO),
 #ifdef O_GMP
   ADD(FUNCTOR_rational1,	ar_rational, 0),
+  ADD(FUNCTOR_numerator1,	ar_numerator, 0),
+  ADD(FUNCTOR_denominator1,	ar_denominator, 0),
   ADD(FUNCTOR_rationalize1,	ar_rationalize, 0),
   ADD(FUNCTOR_rdiv2,		ar_rdiv, 0),
 #endif
@@ -3677,6 +4003,7 @@ static const ar_funcdef ar_funcdefs[] = {
   ADD(FUNCTOR_float_fractional_part1, ar_float_fractional_part, F_ISO),
   ADD(FUNCTOR_float_integer_part1, ar_float_integer_part, F_ISO),
   ADD(FUNCTOR_copysign2,	ar_copysign, 0),
+  ADD(FUNCTOR_nexttoward2,	ar_nexttoward, 0),
 
   ADD(FUNCTOR_sqrt1,		ar_sqrt, F_ISO),
   ADD(FUNCTOR_sin1,		ar_sin, F_ISO),
@@ -3764,13 +4091,39 @@ registerBuiltinFunctions()
 }
 
 
+static atom_t float_rounding_names[5] = {0};
+
 void
 initArith(void)
-{ registerBuiltinFunctions();
+{ GET_LD
+  registerBuiltinFunctions();
 
 #ifdef O_INHIBIT_FP_SIGNALS
   fpsetmask(fpgetmask() & ~(FP_X_DZ|FP_X_INV|FP_X_OFL));
 #endif
+
+  const_nan     = strtod("NaN", NULL);
+  const_inf     = strtod("Inf", NULL);
+  const_neg_inf = strtod("-Inf", NULL);
+
+#ifdef O_GMP
+  LD->arith.rat.max_rational_size = (size_t)-1;
+  LD->arith.rat.max_rational_size_action = ATOM_error;
+
+  setPrologFlag("max_rational_size",	    FT_INTEGER, -1);
+  setPrologFlag("max_rational_size_action", FT_ATOM,    "error");
+#endif
+
+  LD->arith.f.flags = FLT_ROUND_NEAREST|FLT_UNDERFLOW;
+  setPrologFlag("float_overflow",  FT_ATOM, "error");
+  setPrologFlag("float_zero_div",  FT_ATOM, "error");
+  setPrologFlag("float_undefined", FT_ATOM, "error");
+  setPrologFlag("float_underflow", FT_ATOM, "ignore");
+  setPrologFlag("float_rounding",  FT_ATOM, "to_nearest");
+  float_rounding_names[FLT_ROUND_NEAREST]     = ATOM_nearest;
+  float_rounding_names[FLT_ROUND_TO_POS]      = ATOM_to_positive;
+  float_rounding_names[FLT_ROUND_TO_NEG]      = ATOM_to_negative;
+  float_rounding_names[FLT_ROUND_TO_ZERO]     = ATOM_to_zero;
 }
 
 
@@ -3789,6 +4142,146 @@ cleanupArith(void)
   }
 }
 
+int
+is_arith_flag(atom_t k)
+{ return (
+#ifdef O_GMP
+           k == ATOM_max_rational_size ||
+	   k == ATOM_max_rational_size_action ||
+#endif
+	   k == ATOM_float_overflow ||
+	   k == ATOM_float_zero_div ||
+	   k == ATOM_float_undefined ||
+	   k == ATOM_float_underflow ||
+	   k == ATOM_float_rounding );
+}
+
+int
+get_arith_flag(term_t val, atom_t k ARG_LD)
+{ atom_t a;
+#ifdef O_GMP
+  size_t sz;
+
+  if ( k == ATOM_max_rational_size &&
+       (sz=LD->arith.rat.max_rational_size) != (size_t)-1 )
+    return PL_unify_uint64(val, sz);
+  if ( k == ATOM_max_rational_size_action )
+    return PL_unify_atom(val, LD->arith.rat.max_rational_size_action);
+#endif
+  if ( k == ATOM_float_overflow )
+    a = LD->arith.f.flags & FLT_OVERFLOW ? ATOM_infinity : ATOM_error;
+  else if ( k == ATOM_float_zero_div )
+    a = LD->arith.f.flags & FLT_ZERO_DIV ? ATOM_infinity : ATOM_error;
+  else if ( k == ATOM_float_undefined )
+    a = LD->arith.f.flags & FLT_UNDEFINED ? ATOM_nan : ATOM_error;
+  else if ( k == ATOM_float_underflow )
+    a = LD->arith.f.flags & FLT_UNDERFLOW ? ATOM_ignore : ATOM_error;
+  else if ( k == ATOM_float_rounding )
+    a = float_rounding_names[LD->arith.f.flags&FLT_ROUND_MASK];
+  else
+    return FALSE;
+
+  return PL_unify_atom(val, a);
+}
+
+#ifdef O_GMP
+static int
+set_restraint(term_t t, size_t *valp)
+{ GET_LD
+  atom_t inf;
+
+  if ( PL_get_atom(t, &inf) && inf == ATOM_infinite )
+  { *valp = (size_t)-1;
+    return TRUE;
+  }
+  return PL_get_size_ex(t, valp);
+}
+
+static int
+set_restraint_action(term_t t, atom_t key, atom_t *valp ARG_LD)
+{ atom_t act;
+
+  if ( PL_get_atom_ex(t, &act) )
+  { if ( act == ATOM_error )
+    { ok:
+      *valp = act;
+      return TRUE;
+    }
+
+    if ( key == ATOM_max_rational_size_action &&
+	 ( act == ATOM_float ) )
+      goto ok;
+
+    return PL_domain_error("max_rational_size_action", t);
+  }
+
+  return FALSE;
+}
+#endif
+
+
+static void
+set_rounding(int mode)
+{ static int rounding_mode[5] =
+  {0, FE_TONEAREST, FE_UPWARD, FE_DOWNWARD, FE_TOWARDZERO};
+
+  fesetround(rounding_mode[mode]);
+}
+
+
+int
+set_arith_flag(term_t val, atom_t key ARG_LD)
+{ atom_t a;
+
+#ifdef O_GMP
+  if ( key == ATOM_max_rational_size )
+    return set_restraint(val, &LD->arith.rat.max_rational_size);
+  if ( key == ATOM_max_rational_size_action )
+    return set_restraint_action(
+	       val, key,
+	       &LD->arith.rat.max_rational_size_action PASS_LD);
+#endif
+
+  if ( PL_get_atom_ex(val, &a) )
+  { if ( key == ATOM_float_overflow )
+    { if (      a == ATOM_error    ) clear(&(LD->arith.f), FLT_OVERFLOW);
+      else if ( a == ATOM_infinity )   set(&(LD->arith.f), FLT_OVERFLOW);
+      else goto dom;
+    } else if ( key == ATOM_float_zero_div )
+    { if (      a == ATOM_error    ) clear(&(LD->arith.f), FLT_ZERO_DIV);
+      else if ( a == ATOM_infinity )   set(&(LD->arith.f), FLT_ZERO_DIV);
+      else goto dom;
+    } else if ( key == ATOM_float_undefined )
+    { if (      a == ATOM_error    ) clear(&(LD->arith.f), FLT_UNDEFINED);
+      else if ( a == ATOM_nan )        set(&(LD->arith.f), FLT_UNDEFINED);
+      else goto dom;
+    } else if ( key == ATOM_float_underflow )
+    { if (      a == ATOM_error    ) clear(&(LD->arith.f), FLT_UNDERFLOW);
+      else if ( a == ATOM_ignore )     set(&(LD->arith.f), FLT_UNDERFLOW);
+      else goto dom;
+    } else if ( key == ATOM_float_rounding )
+    { int i;
+
+      for(i=1; i<=FLT_ROUND_TO_ZERO; i++)
+      { if ( float_rounding_names[i] == a )
+	{ clear(&(LD->arith.f), FLT_ROUND_MASK);
+	  LD->arith.f.flags |= i;
+	  set_rounding(i);
+	  return TRUE;
+	}
+      }
+      goto dom;
+    } else
+      return FALSE;
+
+    return TRUE;
+
+  dom:
+    return PL_domain_error("flag_value", val);
+  }
+
+  return FALSE;
+}
 
 #if O_COMPILE_ARITH
 
@@ -3933,6 +4426,7 @@ BeginPredDefs(arith)
   PRED_DEF("divmod", 4, divmod, 0)
   PRED_DEF("nth_integer_root_and_remainder", 4,
 	   nth_integer_root_and_remainder, 0)
+  PRED_DEF("rational", 3, rational, 0)
 #endif
   PRED_DEF("set_random", 1, set_random, 0)
 #ifdef O_RANDOM_STATE
