@@ -40,6 +40,7 @@ throw(error(<Formal>, <SWI-Prolog>))
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
 #include "pl-incl.h"
+#include "os/pl-cstack.h"
 /* BeOS has EACCES defined elsewhere, but errno is here */
 #if !defined(EACCES) || defined(__BEOS__)
 #include <errno.h>
@@ -229,6 +230,19 @@ PL_error(const char *pred, int arity, const char *msg, PL_error_code id, ...)
 			   PL_ATOM, ATOM_float_underflow);
       break;
     }
+    case ERR_AR_TRIPWIRE:
+    { atom_t tripwire = va_arg(args, atom_t);
+      Number num      = va_arg(args, Number);
+      term_t actual   = PL_new_term_ref();
+
+      rc = (_PL_put_number(actual, num) &&
+	    PL_unify_term(formal,
+			  PL_FUNCTOR, FUNCTOR_resource_error1,
+			    PL_FUNCTOR, FUNCTOR_tripwire2,
+			      PL_ATOM, tripwire,
+			      PL_TERM, actual));
+      break;
+    }
     case ERR_DOMAIN:			/*  ERR_INSTANTIATION if var(arg) */
     { atom_t domain = va_arg(args, atom_t);
       term_t arg    = va_arg(args, term_t);
@@ -675,6 +689,7 @@ PL_error(const char *pred, int arity, const char *msg, PL_error_code id, ...)
       break;
     }
     default:
+      rc = FALSE;
       assert(0);
   }
   va_end(args);
@@ -895,12 +910,13 @@ printMessage(atom_t severity, ...)
     { rc = PL_call_predicate(NULL, PL_Q_NODEBUG|PL_Q_PASS_EXCEPTION,
 			     pred, av);
     } else if ( LD->in_print_message <= OK_RECURSIVE*2 )
-    { Sfprintf(Serror, "Message: ");
+    { Sfprintf(Serror, "print_message/2: recursive call: ");
       if ( ReadingSource )
 	Sfprintf(Serror, "%s:%d ",
 		 PL_atom_chars(source_file_name), (int)source_line_no);
       rc = PL_write_term(Serror, av+1, 1200, 0);
       Sfprintf(Serror, "\n");
+      PL_backtrace(5, 1);
     } else				/* in_print_message == 2 */
     { Sfprintf(Serror, "printMessage(): recursive call\n");
     }
