@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2019, University of Amsterdam,
+    Copyright (c)  1985-2020, University of Amsterdam,
                               VU University Amsterdam,
 			      CWI, Amsterdam
     All rights reserved.
@@ -138,7 +138,11 @@ COMMON(void)		freeArithLocalData(PL_local_data_t *ld);
 COMMON(int)		ar_sign_i(Number n1);
 COMMON(int)		ar_signbit(Number n1);
 COMMON(int)		check_float(double f);
+COMMON(int)		ar_rdiv_mpz(Number n1, Number n2, Number r);
 COMMON(int)		PL_eval_expression_to_int64_ex(term_t t, int64_t *val);
+COMMON(int)		is_arith_flag(atom_t k);
+COMMON(int)		get_arith_flag(term_t val, atom_t k ARG_LD);
+COMMON(int)		set_arith_flag(term_t val, atom_t k ARG_LD);
 
 /* pl-bag.c */
 COMMON(void)		markAtomsFindall(PL_local_data_t *ld);
@@ -465,6 +469,7 @@ COMMON(word)		pl_prolog_flag5(term_t key, term_t value,
 					control_t h);
 COMMON(int)		setDoubleQuotes(atom_t a, unsigned int *flagp);
 COMMON(int)		setBackQuotes(atom_t a, unsigned int *flagp);
+COMMON(int)		setRationalSyntax(atom_t a, unsigned int *flagp);
 COMMON(void)		initPrologFlags(void);
 COMMON(void)		cleanupPrologFlags(void);
 
@@ -527,10 +532,8 @@ COMMON(void)		reconsultFinalizePredicate(sf_reload *rl, Definition def,
 COMMON(void)		destroyDefinition(Definition def);
 COMMON(Procedure)	resolveProcedure__LD(functor_t f, Module module ARG_LD);
 COMMON(Definition)	trapUndefined(Definition undef ARG_LD);
-COMMON(word)		pl_retractall(term_t head);
 COMMON(word)		pl_abolish(term_t atom, term_t arity);
 COMMON(word)		pl_abolish1(term_t pred);
-COMMON(word)		pl_get_predicate_attribute(term_t pred, term_t k, term_t v);
 COMMON(int)		redefineProcedure(Procedure proc, SourceFile sf,
 					  unsigned int suppress);
 COMMON(word)		pl_index(term_t pred);
@@ -559,7 +562,6 @@ COMMON(void)		unlinkSourceFileModule(SourceFile sf, Module m);
 COMMON(void)		addProcedureSourceFile(SourceFile sf, Procedure proc);
 COMMON(int)		hasProcedureSourceFile(SourceFile sf, Procedure proc);
 COMMON(int)		reloadHasClauses(SourceFile sf, Procedure proc ARG_LD);
-COMMON(int)		isDefinedProcedureSource(Procedure proc);
 COMMON(ClauseRef)	assertProcedureSource(SourceFile sf, Procedure proc,
 					      Clause clause ARG_LD);
 COMMON(int)		setAttrProcedureSource(SourceFile sf, Procedure proc,
@@ -581,7 +583,7 @@ COMMON(int)		atom_varnameW(const pl_wchar_t *s, size_t len);
 COMMON(int)		atom_is_named_var(atom_t name);
 COMMON(strnumstat)	str_number(const unsigned char *string,
 				   unsigned char **end,
-				   Number value, bool escape);
+				   Number value, int flags);
 COMMON(const char *)	str_number_error(strnumstat rc);
 COMMON(word)		pl_raw_read(term_t term);
 COMMON(word)		pl_raw_read2(term_t stream, term_t term);
