@@ -441,7 +441,6 @@ struct PL_local_data
 #endif
   int		in_arithmetic;		/* doing arithmetic */
   int		in_print_message;	/* Inside printMessage() */
-  int		autoload_nesting;	/* Nesting level in autoloader */
   gen_t		gen_reload;		/* reload generation */
   void *	glob_info;		/* pl-glob.c */
   IOENC		encoding;		/* default I/O encoding */
@@ -587,7 +586,14 @@ struct PL_local_data
     { gmp_randstate_t state;
       int initialised;
     } random;
+    struct
+    { size_t max_rational_size;
+      atom_t max_rational_size_action;
+    } rat;
 #endif
+    struct
+    { unsigned int flags;		/* FTL_* */
+    } f;
   } arith;
 
 #if O_CYCLIC
@@ -605,6 +611,14 @@ struct PL_local_data
     int in_answer_completion;		/* Running answer completion */
     term_t delay_list;			/* Global delay list */
     term_t idg_current;			/* Current node in IDG (trie symbol) */
+    struct
+    { atom_t max_table_subgoal_size_action;
+      size_t max_table_subgoal_size;
+      atom_t max_table_answer_size_action;
+      size_t max_table_answer_size;
+      atom_t max_answers_for_subgoal_action;
+      size_t max_answers_for_subgoal;
+    } restraint;
   } tabling;
 
   struct
@@ -705,6 +719,11 @@ struct PL_local_data
     int64_t	cgc_inferences;		/* Inferences at last cgc consider */
   } clauses;
 
+  struct
+  { DefinitionChain nesting;		/* Nesting chain in the autoloader */
+    Definition	loop;			/* We are looping on this def */
+  } autoload;
+
   struct
   { intptr_t _total_marked;		/* # marked global cells */
     intptr_t _trailcells_deleted;	/* # garbage trailcells */
