@@ -3,8 +3,9 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2017, University of Amsterdam
+    Copyright (c)  1985-2020, University of Amsterdam
                               VU University Amsterdam
+			      CWI, Amsterdam
     All rights reserved.
 
     Redistribution and use in source and binary forms, with or without
@@ -84,7 +85,7 @@ _lookupModule(atom_t name ARG_LD)
 #endif
   set(m, M_CHARESCAPE);
   if ( !GD->options.traditional )
-    set(m, DBLQ_STRING|BQ_CODES);
+    set(m, DBLQ_STRING|BQ_CODES|O_RATIONAL_SYNTAX);
 
   if ( name == ATOM_user || name == ATOM_system )
     m->procedures = newHTable(PROCEDUREHASHSIZE);
@@ -564,11 +565,17 @@ set_module(Module m, term_t prop ARG_LD)
       { m->class = class;
 	return TRUE;
       } else if ( class == ATOM_temporary )
-      { if ( m->procedures && m->procedures->size != 0 )
-	  return PL_error(NULL, 0,
-			  "module is not empty",
+      { Table procs;
+
+	if ( m->class == ATOM_user &&
+	     !((procs=m->procedures) && procs->size != 0) )
+	{ m->class = class;
+	} else
+	{ return PL_error(NULL, 0,
+			  m->class != ATOM_user ? "Not a user module" :
+						  "module is not empty",
 			  ERR_PERMISSION, ATOM_module_property, ATOM_class, arg);
-	m->class = class;
+	}
 	return TRUE;
       } else
 	return PL_error(NULL, 0, NULL, ERR_DOMAIN, ATOM_module_class, arg);
@@ -1549,7 +1556,7 @@ retry:
       } else
       { lingerDefinition(odef);
       }
-      set(old, pflags);
+      set(old, pflags|PROC_IMPORTED);
 
       succeed;
     }
