@@ -76,6 +76,8 @@ resetProlog(int clear_stacks)
   depth_limit   = (uintptr_t)DEPTH_NO_LIMIT;
 #endif
 
+  LD->autoload.nesting = NULL;
+  LD->autoload.loop = NULL;
   updateAlerted(LD);
 
   return TRUE;
@@ -703,6 +705,8 @@ last_arg:
 #ifdef O_GMP
     if ( isMPZNum(*p) )
       return 0x62f8da3c;		/* TBD: make key from MPZ */
+    if ( isMPQNum(*p) )
+      return 0xed7ef4ea;		/* TBD: make key from MPQ */
 #endif
     printk(context, "Illegal indirect datatype");
     return key;
