@@ -3,7 +3,7 @@
     Author:        Jan Wielemaker
     E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (c)  1985-2019, University of Amsterdam
+    Copyright (c)  1985-2020, University of Amsterdam
                               VU University Amsterdam
 			      CWI, Amsterdam
     All rights reserved.
@@ -38,6 +38,7 @@
 #include "pl-incl.h"
 #include "pl-dbref.h"
 #include "pl-event.h"
+#include "pl-tabling.h"
 
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 General  handling  of  procedures:  creation;  adding/removing  clauses;
@@ -176,6 +177,11 @@ destroyDefinition(Definition def)
 #endif
   }
 
+  if ( def->tabling )
+  { freeHeap(def->tabling, sizeof(*def->tabling));
+    def->tabling = NULL;
+  }
+
   DEBUG(MSG_CGC_PRED,
 	Sdprintf("destroyDefinition(%s)\n", predicateName(def)));
   if ( true(def, P_DIRTYREG) )
@@ -275,6 +281,8 @@ resetProcedure(Procedure proc, bool isnew)
   def->impl.clauses.number_of_clauses = 0;
   if ( def->events )
     destroy_event_list(&def->events);
+  if ( def->tabling )
+    tbl_reset_tabling_attributes(def);
 
   if ( isnew )
   { deleteIndexes(&def->impl.clauses, TRUE);
@@ -363,7 +371,13 @@ overruleImportedProcedure(Procedure proc, Module target)
 { GET_LD
   Definition def = getProcDefinition(proc);
 
-  assert(def->module != target);	/* e.g., imported */
+  if ( true(def, P_AUTOLOAD) )
+    return PL_error(NULL, 0, NULL, ERR_PERMISSION_PROC,
+		    ATOM_redefine, ATOM_imported_procedure, proc);
+
+  if ( def->module == target )
+    return TRUE;
+
   if ( true(def->module, M_SYSTEM) )
   { return PL_error(NULL, 0, NULL, ERR_PERMISSION_PROC,
 		    ATOM_redefine, ATOM_built_in_procedure, proc);
@@ -403,12 +417,8 @@ lookupProcedureToDefine(functor_t def, Module m)
   Procedure proc;
 
   if ( (proc = isCurrentProcedure(def, m)) )
-  { Definition def = getProcDefinition(proc);
-
-    if ( def->module != m )
-    { if ( !overruleImportedProcedure(proc, m) )
-	return NULL;
-    }
+  { if ( !overruleImportedProcedure(proc, m) )
+      return NULL;
 
     return proc;
   }
@@ -753,6 +763,17 @@ typedef struct
 } cur_enum;
 
 
+static int
+isDefinedOrAutoloadProcedure(Procedure proc)
+{ Definition def = proc->definition;
+
+  if ( true(def, PROC_DEFINED|P_AUTOLOAD) )
+    succeed;
+
+  return hasClausesDefinition(def) ? TRUE : FALSE;
+}
+
+
 static Procedure
 visibleProcedure(functor_t f, Module m ARG_LD)
 { ListCell c;
@@ -761,7 +782,7 @@ visibleProcedure(functor_t f, Module m ARG_LD)
   for(;;)
   { next:
 
-    if ( (p = isCurrentProcedure(f, m)) && isDefinedProcedure(p) )
+    if ( (p = isCurrentProcedure(f, m)) && isDefinedOrAutoloadProcedure(p) )
       return p;
 
     for(c=m->supers; c; c=c->next)
@@ -2350,6 +2371,30 @@ found:
 }
 
 
+static int
+test_autoload_loop(Definition def ARG_LD)
+{ DefinitionChain ch;
+
+  for(ch=LD->autoload.nesting; ch; ch=ch->next)
+  { if ( ch->definition == def )
+    { LD->autoload.loop = def;
+
+      Sdprintf("ERROR: autoload loop:\n");
+      Sdprintf("ERROR:   %s\n", predicateName(def));
+
+      for(ch=LD->autoload.nesting; ch; ch=ch->next)
+      { Sdprintf("ERROR:   %s\n", predicateName(ch->definition));
+	if ( ch->definition == def )
+	  break;
+      }
+
+      return FALSE;
+    }
+  }
+
+  return TRUE;
+}
+
 /* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 Call the autoloader for the given definition.
 - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
@@ -2361,6 +2406,7 @@ autoLoader(Definition def)
   term_t argv;
   qid_t qid;
   atom_t answer = ATOM_nil;
+  struct definition_chain cell;
 
   if ( !GD->procedures.undefinterc4 )
     GD->procedures.undefinterc4 = PL_pred(FUNCTOR_undefinterc4,
@@ -2375,14 +2421,18 @@ autoLoader(Definition def)
   PL_put_integer( argv+2, def->functor->arity);
 
   push_input_context(ATOM_autoload);
-  LD->autoload_nesting++;
-  if ( (qid = PL_open_query(MODULE_system, PL_Q_NODEBUG,
+  cell.definition = def;
+  cell.next = LD->autoload.nesting;
+  LD->autoload.nesting = &cell;
+  if ( (qid = PL_open_query(MODULE_system, PL_Q_NODEBUG|PL_Q_PASS_EXCEPTION,
 			    GD->procedures.undefinterc4, argv)) )
   { if ( PL_next_solution(qid) )
       PL_get_atom(argv+3, &answer);
     PL_close_query(qid);
+  } else if ( PL_exception(0) )
+  { PL_clear_exception();
   }
-  LD->autoload_nesting--;
+  LD->autoload.nesting = LD->autoload.nesting->next;
   pop_input_context();
   PL_discard_foreign_frame(cid);
 
@@ -2415,11 +2465,8 @@ trapUndefined(Definition def ARG_LD)
 
 					/* Trap via exception/3 */
   if ( truePrologFlag(PLFLAG_AUTOLOAD) && !GD->bootsession )
-  { if ( LD->autoload_nesting > 100 )
-    { LD->autoload_nesting = 1;
-      sysError("trapUndefined(): undefined: %s", predicateName(def));
-
-      return def;
+  { if ( !test_autoload_loop(def PASS_LD) )
+    { return def;
     } else
     { atom_t answer = autoLoader(def);
 
@@ -2637,9 +2684,10 @@ allVars(int argc, Word argv ARG_LD)
 }
 
 
-word
-pl_retractall(term_t head)
+static
+PRED_IMPL("retractall", 1, retractall, PL_FA_NONDETERMINISTIC|PL_FA_ISO)
 { GET_LD
+  term_t head = A1;
   term_t thehead = PL_new_term_ref();
   Procedure proc;
   Definition def;
@@ -2715,6 +2763,11 @@ pl_retractall(term_t head)
 	{ if ( PL_exception(0) )
 	    break;
 	}
+      } else
+      { if ( PL_exception(0) )
+	{ rc = FALSE;
+	  break;
+	}
       }
 
       PL_rewind_foreign_frame(fid);
@@ -2821,7 +2874,6 @@ static const patt_mask patt_masks[] =
   { ATOM_spy,		   SPY_ME },
   { ATOM_tabled,	   P_TABLED },
   { ATOM_incremental,	   P_INCREMENTAL },
-  { ATOM_abstract,	   P_ABSTRACT },
   { ATOM_tshared,	   P_TSHARED },
   { ATOM_trace,		   TRACE_ME },
   { ATOM_hide_childs,	   HIDE_CHILDS },
@@ -2835,6 +2887,7 @@ static const patt_mask patt_masks[] =
   { ATOM_non_terminal,	   P_NON_TERMINAL },
   { ATOM_quasi_quotation_syntax, P_QUASI_QUOTATION_SYNTAX },
   { ATOM_clausable,	   P_CLAUSABLE },
+  { ATOM_autoload,	   P_AUTOLOAD },
   { (atom_t)0,		   0 }
 };
 
@@ -2885,10 +2938,13 @@ num_visible_clauses(Definition def, atom_t key)
 }
 
 
-word
-pl_get_predicate_attribute(term_t pred,
-			   term_t what, term_t value)
-{ GET_LD
+static
+PRED_IMPL("$get_predicate_attribute", 3, get_predicate_attribute,
+	  PL_FA_TRANSPARENT)
+{ PRED_LD
+  term_t pred  = A1;
+  term_t what  = A2;
+  term_t value = A3;
   Procedure proc;
   Definition def;
   functor_t fd;
@@ -2923,7 +2979,7 @@ pl_get_predicate_attribute(term_t pred,
   } else if ( key == ATOM_defined )
   { int d;
 
-    if ( isDefinedProcedureSource(proc) )
+    if ( isDefinedProcedure(proc) )
       d = 1;
     else
       d = 0;
@@ -2977,6 +3033,12 @@ pl_get_predicate_attribute(term_t pred,
     if ( def->impl.clauses.number_of_clauses == 0 && false(def, P_DYNAMIC) )
       fail;
     return PL_unify_integer(value, num_visible_clauses(def, key));
+  } else if ( tbl_is_predicate_attribute(key) )
+  { size_t sz_value;
+
+    if ( tbl_get_predicate_attribute(def, key, &sz_value) == TRUE )
+      return PL_unify_int64(value, sz_value);
+    return FALSE;
   } else if ( (att = attribute_mask(key)) )
   { return PL_unify_integer(value, (def->flags & att) ? 1 : 0);
   } else
@@ -3127,8 +3189,24 @@ PRED_IMPL("$set_predicate_attribute", 3, set_predicate_attribute,
   int val;
   uintptr_t att;
 
-  if ( !PL_get_atom_ex(what, &key) ||
-       !get_bool_or_int_ex(value, &val PASS_LD) ||
+  if ( !PL_get_atom_ex(what, &key) )
+    return FALSE;
+  if ( tbl_is_predicate_attribute(key) )
+  { size_t v;
+    atom_t inf;
+
+    if ( PL_get_atom(value, &inf) && inf == ATOM_infinite )
+      v	= (size_t)-1;
+    else if ( !PL_get_size_ex(value, &v) )
+      return FALSE;
+
+    if ( get_procedure(pred, &proc, 0, GP_DEFINE|GP_NAMEARITY) )
+      return tbl_set_predicate_attribute(proc->definition, key, v) == TRUE;
+
+    return FALSE;
+  }
+
+  if ( !get_bool_or_int_ex(value, &val PASS_LD) ||
        !(att = attribute_mask(key)) )
     return FALSE;
 
@@ -3512,8 +3590,11 @@ pl_list_generations(term_t desc)
 		 *******************************/
 
 BeginPredDefs(proc)
+  PRED_DEF("retractall", 1, retractall, PL_FA_NONDETERMINISTIC|PL_FA_ISO)
   PRED_DEF("$set_predicate_attribute", 3, set_predicate_attribute,
 	   PL_FA_TRANSPARENT)
+  PRED_DEF("$get_predicate_attribute", 3, get_predicate_attribute,
+	   PL_FA_TRANSPARENT)
   PRED_DEF("$default_predicate", 2, default_predicate, PL_FA_TRANSPARENT)
   PRED_DEF("meta_predicate", 1, meta_predicate, PL_FA_TRANSPARENT)
   PRED_DEF("$get_clause_attribute", 3, get_clause_attribute, 0)
